#!/usr/bin/perl
#  export.pl - functions to implement exporting data to other formats
#
#  Written by Curtis Olson.  Started October 19, 1994.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# export.pl,v 0.9 2001/01/17 20:26:02 genemagnier Exp


package CBB;

use strict;   # don't take no guff

# save all the precious data to a file
sub export_cbb9 {
    # in: filename (including .cbb extension)
    # out: result

    my($file) = "$_[0]";
    my($auto_save_file, $key);
    my(@trans);
    my($file_exists) = 0;
    my($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, 
       $ctime, $blksize, $blocks);

    print DEBUG "the file is: $_[0]\n" if $CBB::debug;
    print DEBUG "Saving the cbb format file: $file\n" if $CBB::debug;

    if ($CBB::calced == 0) { &calc_trans(); }
    if ($CBB::sorted_keys == 0) { &sort_keys(); }

    if ( $CBB::encrypt ne "" ) {
	open(SAVE, "|$CBB::encrypt > $file.new") || return "error";
    } else {
    	open(SAVE, ">$file.new") || return "error";
    }

    # Print some header stuff
    printf (SAVE "# CBB-Account File\n");
    printf (SAVE "#\n");
    printf (SAVE "# CBB Version = $CBB::version_num\n");
    printf (SAVE "# GROUP = $CBB::group\n"); # 
    printf (SAVE "# Current Balance = %.2f\n", $CBB::BALS{Current});
    printf (SAVE "# Ending Balance = %.2f\n", $CBB::BALS{Amount});
    printf (SAVE "# Transaction Count = $CBB::BALS{Count}\n");
    printf (SAVE "# Cleared Balance = %.2f\n", $CBB::BALS{Xamount});
    printf (SAVE "# Cleared Txn Count = $CBB::BALS{Xcount}\n");
    printf (SAVE "# Date Saved = %s\n", &raw_date());
    printf (SAVE "# User = $CBB::user_name\n");
    printf (SAVE "#\n");
    printf (SAVE "# date  check  desc  debit  credit  cat  com  cleared\n");
    printf (SAVE "# ---------------------------------------------------\n");

    foreach $key (@CBB::KEYS) {
	# strip off last total
	@trans = split(/\t/, $CBB::TRANS{$key});
	print SAVE join ("\t", @trans[0..7]) . "\n";
    }

    close(SAVE);

    # preserve file permissions if the file exists
    if ( -e $file ) {
	$file_exists = 1;
	($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, 
	 $ctime, $blksize, $blocks) = stat($file);
	print DEBUG "file permissions = $mode\n" if $CBB::debug;
	print DEBUG "file owner = $uid  group = $gid\n" if $CBB::debug;
    }

    if ( $file_exists ) {
	unlink("$file.bak");
	rename("$file", "$file.bak");
    }

    rename("$file.new", "$file");

    if ( $file_exists ) {
	chown($uid, $gid, $file);
	chmod($mode, $file);
    }

    return "ok";
}

# save data in the CBB format
sub export_cbb8 {
    # in: filename
    # out: result

    my($file) = @_;
    my($key);

    if ($CBB::calced == 0) {
	&calc_trans();
    }

    open(SAVE, ">$file.new");

    if ($CBB::sorted_keys == 0) {
	&sort_keys();
    }

    foreach $key (@CBB::KEYS) {
	print (SAVE "$CBB::TRANS{$key}\n");
    }

    close(SAVE);

    unlink("$file.bak");
    rename("$file", "$file.bak");
    rename("$file.new", "$file");

    return "ok";
}


# Contributed by Christopher Browne, Oct. 18/94
# export a quicken export file (.qif)
sub export_qif {
    # in: file
    # out: result
    
    my($file) = @_;
    my($key);
    my($date, $check, $desc, $debit, $credit, $cat, $split, $com, $cleared);
    my($amount, $i, @SPLIT, $scom, $scat, $junk);
    my($dy, $mo, $yr);

    $CBB::sorted_keys = 0;
    $CBB::calced = 0;
    
    open(QIF, ">$file");
    
    print QIF "Type:Bank\n";      # This will need to change,
    # eventually, to handle other
    # varieties of QUICKEN accounts.
    # Credit cards, investments, etc.
    # Later.

    ($date, $check, $desc, $debit, $credit, $cat, $split, $com,
     $cleared) = ("", "", "", "", "", "", "", "", "");

    foreach $key (keys (%CBB::TRANS)) {
        ($date, $check, $desc, $debit, $credit, $cat, $com, $cleared,
	 $junk) = split(/\t/, $CBB::TRANS{$key});
     
	# Handle date
	$yr = substr($date, 2, 2);
	$mo = substr($date, 4, 2);
	$dy = substr($date, 6, 2);
	printf QIF "D%d/%2d/%2d\n", $mo, $dy, $yr;
     
	# Handle amount
	$amount = $credit-$debit;
	printf QIF "T%.2lf\n", $amount;

	printf QIF "C$cleared\n" unless ($cleared eq "");
     
	# Handle Ref. #
	print QIF "N$check\n" unless ($check eq "");
	print QIF "P$desc\n" unless ($desc eq "");

	# Replace underscores by blanks in transfer categories. (BW)  
	while (($cat =~ s/\[([^\]_]*)_([^\]]*)\]/\[$1 $2\]/g) != 0) {
	}

	print QIF "L$cat\n" unless (substr($cat, 0, 1) eq "\|");  # split txn
	# Handle splitting of txn
	if (substr($cat, 0, 1) eq "\|")	{
	    # Take: |Salary|2434.70|Tax-Fed|-0.34|Tax-FICA|-33.25|Tax-State|-5.78|
	    @SPLIT = split(/\|/, $cat);
	    print QIF "L$SPLIT[1]\n";   
	    # Pretend that the initial category is "the one"
	    for ($i = 1 ; $i <= $#SPLIT ; $i += 3) {
		$scat = $SPLIT[$i];
		print QIF "S$scat\n";
		$scom = $SPLIT[$i+1];
		print QIF "E$scom\n";
		$amount = $SPLIT[$i+2];
		print QIF "\$$amount\n";
	    }
	} else {
	}
	print QIF "M$com\n" unless ($com eq "");
	print QIF  "^\n";
    }
    
    close(QIF);

    return "ok";
}


1;				# need to return a true value
