#!/usr/bin/perl
#  memorized.pl - functions to implement memorized transactions
#
#  Written by Curtis Olson.  Started October 13, 1994.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# memorized.pl,v 0.9 2001/01/17 20:26:02 genemagnier Exp

package CBB;

use strict;    # don't take no guff


# WARNING:  This file must be "required" after the main transaction file.

# %CBB::MEMS - an associative array of transactions and description keys
# @CBB::MEMKEYS - a sorted list of description keys (for memorized transactions)


# initialize the memorized transaction list
sub init_mems {
    # out: result

    %CBB::MEMS = ();
    $CBB::sorted_memkeys = 1;

    return "ok";
}


# insert a memorized transaction into the MEMS list
sub insert_mem {
    # in: trans
    # out: trans

    my($trans) = @_;
    my($date, $check, $desc, $debit, $credit, $cat, $com, $cleared, $total) =
	split(/\t/, $trans);

    $CBB::MEMS{$desc} = $trans;

    print DEBUG "insert_mem:  $desc -> $trans\n" if $CBB::debug;

    return "$trans";
}


# insert a memorized transaction into the MEMS list *AND* rebuild the MEMKEYS
sub insert_and_update_mem {
    # in: trans
    # out: trans

    my($trans) = @_;
    my($date, $check, $desc, $debit, $credit, $cat, $com, $cleared, $total) =
	split(/\t/, $trans);

    $CBB::MEMS{$desc} = $trans;
    @CBB::MEMKEYS = sort(keys %CBB::MEMS);

    print DEBUG "insert_mem:  $desc -> $trans\n" if $CBB::debug;

    return "$trans";
}


# build the MEMS list
sub rehash_mems {
    # out: result

    my($key);

    &init_mems();

    if ($CBB::calced == 0) {
	&calc_trans();
    }

    if ($CBB::sorted_keys == 0) {
	&sort_keys();
    }

    foreach $key (@CBB::KEYS) {
	&insert_mem($CBB::TRANS{$key});
    }

    @CBB::MEMKEYS = sort(keys %CBB::MEMS);

    return "ok";
}


# attempt to find a transaction matching the description
# incomplete descriptions are allowed
# returns 2 lines: 
#  <keyword> match match match
#  VALUE <Nth matched entry>
# KEYWORD: UNIQUE PARTIAL COMPLET NONE
sub find_mem {
    # in: desc
    # out: trans

    my($nkey, $desc) = split ('\t', $_[0]);
    my(@result, $count, $i, $match, $memkey, $words);

    if ( $desc ne "" ) {
	$count = 0;
	$match = 0;

        foreach $memkey (@CBB::MEMKEYS) {
	    if ( $memkey =~ m/^$desc/i ) {
		print DEBUG "$memkey <=> $desc\n" if $CBB::debug;
		$count++;

		if ( $memkey =~ m/^$desc$/i ) {
		    print DEBUG "exact match\n" if $CBB::debug;
		    $match = 1;
		}
		@result = (@result, $memkey);
	    }
        }
	if ($count && ($nkey < $count)) {
	    for ($i = 0; $i < @result; $i++) {
		$words = $words . " " . $result[$i];
	    }
	    if (($count == 1) && $match) { 
		return "EXACT\t$words\nVALUE\t$CBB::MEMS{$result[$nkey]}";
	    } 
	    if ($count == 1) { 
		return "UNIQUE\t$words\nVALUE\t$CBB::MEMS{$result[$nkey]}";
	    } 
	    if ( $match ) {
		return "COMPLETE\t$words\nVALUE\t$CBB::MEMS{$result[$nkey]}";
	    } 
	    return "PARTIAL\t$words\nVALUE\t$CBB::MEMS{$result[$nkey]}";
	}
    }

    # need to have the return -- cbb looks for second line
    return "NONE\t \n";
}


&init_mems();


1;				# need to return a true value
