#! /usr/local/bin/perl
#  rep-txn-list.pl - Prints a report of the transactions sorted by date
#
#  Written by Curtis Olson.  Started November 12, 1994.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# rep-txn-list.pl.in,v 1.2 2001/07/13 04:05:59 briank Exp

# specify the installed location of the necessary pieces.
my($cbbpath) = "/usr/local/lib/cbb-0.9";
@INC = ("$cbbpath/perl", @INC);
package CBB;

use strict;    # don't take no guff

my($account, @account_list, $name, $key, $trans, %ALLTRANS);
my($gtotal, $junk, $result);
my($date, $date_fmt);

require "common.pl";
require "reports.pl";
require "engine.pl";
require "accounts.pl";
require "memorized.pl";

if ($#ARGV < 1) {
    die "Usage: report [ -from yyyymmdd ] [ -to yyyymmdd ] path accounts";
}

# process arguments
my($fromdate, $todate, @account_list) = &process_rep_args();

%ALLTRANS = ();

# load all matching transactions from all specified accounts (ignoring
# those that are outside the specified date range)

foreach $account ( @account_list ) {
    $name = &file_basename($account);

    # open the account
    (&load_trans($account) eq "ok") || die "Cannot open account:  $account";

    $result = &first_trans();
    while ( $result ne "none" ) {
        ($key, $trans) = split(/\t/, $result, 2);
        ($date, $junk) = split(/\t/, $trans, 2);

	# default values of from and to are 0 and 99999999 respectively
        if (($fromdate <= $date) && ($todate >= $date)) {
	    $ALLTRANS{"$key$name"} = $trans;
        }
        $result = &next_trans();
    }
}

# sort and print
foreach $key (sort (keys %ALLTRANS) ) {
    # print $ALLTRANS{$key} . "\n";
    &format_line("$ALLTRANS{$key}");
}


$gtotal = 0.00;

sub format_line {
    my($date, $check, $desc, $debit, $credit, $cat, $com, $cleared,
          $total) = split(/\t/, $_[0]);
    my($year,$mon,$day) = $date =~ /(\d\d\d\d)(\d\d)(\d\d)/;
    my($nicedate, $cutdesc, $cutcom, $nicecat);

    $year = substr($year, 2, 4);

    if ( $date_fmt == 1 ) {
	$nicedate = "$mon/$day/$year";
    } else {
	$nicedate = "$day.$mon.$year";
    }
    $cutdesc = substr($desc, 0, 15);
    $cutcom = substr($com, 0, 15);
    $nicecat = substr($cat, 0, 9);
    if ( substr($cat, 0, 1) eq "|" ) {
        $nicecat = "-Splits-";
    }

    $gtotal += $credit - $debit;

    printf("%5s  %-8s  %-15s  %9.2f  %9.2f  %-1s  %9.2f\n",
          $check, $nicedate, $cutdesc, $debit, $credit, $cleared, $gtotal);
    printf("%5s  %-8s  %-15s  %-9s\n\n", "", "", $cutcom, $nicecat);
}
