#!/usr/bin/wish -f
#  'CBB' -- Check Book Balancer
#
#   bindings.tcl -- common bindings
#
#  Written by Curtis Olson.  Started August 25, 1994.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# bindings.tcl,v 0.10 2001/07/13 04:05:59 briank Exp


#------------------------------------------------------------------------------
# Setup auto highlighting of entry fields
#------------------------------------------------------------------------------
proc setup_auto_hilite {} {
    global cbb

    bind Entry <FocusIn> { 
	set textvar [lindex [%W configure -textvariable] 4]
        set value [subst $[subst $textvar]]
	if { $cbb(debug) } { puts "$textvar = '$value'" }

	if { "$value" == "" } {
	    # do nothing, blank value
	} else {
	    %W selection from 0
	    %W selection to end
	}
    }

    bind Entry <FocusOut> { 
	%W selection clear
    }
}


#------------------------------------------------------------------------------
# Setup default field binding
#------------------------------------------------------------------------------

proc reset_desc_memory { } {
    global cbb

    set cbb(mem_entry) 0
    set cbb(mem_match) ""
    set cbb(mem_partial) ""
}

proc setup_default_binding {} {
    global cbb desc addcat cat

    set tabList { .entry.line1.check .entry.line1.date .entry.line1.desc \
	    .entry.line1.debit .entry.line1.credit .entry.line2.com \
	    .entry.line2.cat .entry.line1.clear }

    foreach field $tabList {
        bind $field <Return> { done_entering }
        bind $field <KP_Enter> { done_entering }
	bind $field <Shift-Tab> { focus [tk_focusPrev %W] }
	bind $field <ISO_Left_Tab> { focus [tk_focusPrev %W] }
    }

    # The following are duplicates from menu.tk where these things
    # are bound to "all".  I am duplicating them here for the widgets
    # that ignore "all" bindings.
    foreach field { .entry.line1.desc .entry.line2.cat } {
	# <Tab> is handled individually to make tab completion possible

    	bind $field <Meta-q> { cbbQuit }
	bind $field <Alt-q> { cbbQuit }
	bind $field <Meta-u> { undoAction }
	bind $field <Alt-u> { undoAction }
	bind $field <Meta-n> { clear_entry_area }
	bind $field <Alt-n> { clear_entry_area }
	bind $field <Meta-e> { 
	    if { [listGetCurTrans] >= 1 } {
		update_entry_area [listGetCurTrans].0
	    }
	}                                                  
	bind $field <Alt-e> { 
	    if { [listGetCurTrans] >= 1 } {
		update_entry_area [listGetCurTrans].0
	    }
	}
	bind $field <Meta-s> { 
	    cbbWindow.splits
	    tkwait window .splits
	}
	bind $field <Alt-s> { 
	    cbbWindow.splits
	    tkwait window .splits
	}
	bind $field <Meta-c> { cbbWindow.catwin }
	bind $field <Alt-c> { cbbWindow.catwin }
    }

    # We bind <FocusOut> on fields where we need to set a dependent value
    # based on the entry value
    bind .entry.line1.date <FocusOut> {
	if { $cbb(debug) } { puts $nicedate}
	set date [nicedate_to_date $nicedate $cbb(cur_date)]
	set nicedate [date_to_nicedate $date]
    }

    bind .entry.line2.cat <FocusOut> {
	if { "$nicecat" != "-Splits-" } {
	    set cat $nicecat
	}
    }

    ######### bind <Control-Tab> to date by hand
    bind .entry.line1.date <Control-Tab> {
	if { $cbb(debug) } { puts $nicedate}
	set date [nicedate_to_date $nicedate $cbb(cur_date)]
	set nicedate [date_to_nicedate $date]
    }

    ######### bind <Tab> to desc by hand
    bind .entry.line1.desc <Tab> {

	# for any of these three cases, we just skip to next entry
	if {! $cbb(use_mems) } { focus [tk_focusNext %W]; return }
	if {! [string length $desc] } { focus [tk_focusNext %W]; return }
	if {! [string compare $desc $cbb(mem_match)] } { focus [tk_focusNext %W]; return }

	set cbb(mem_partial) $desc
	puts $eng "find_mem 0\t$desc"; flush $eng
	gets $eng result
	gets $eng match 
	set cbb(mem_entry) 1
	if { $cbb(debug) } { puts $result; puts $match}
	set pieces [split $result "\t"]
	set stat  [lindex $pieces 0]
	set value [lindex $pieces 1]
	switch $stat {
	    "NONE" {
		.status.line configure -text "no match found."
		set desc $cbb(mem_partial)
		set cbb(mem_match) $desc
		focus [tk_focusNext %W]
		return 
	    } 
	    "PARTIAL" {
		.status.line configure -text "partial match: '$value'"
		update_from_mem $match
	    } 
	    "COMPLETE" {
		.status.line configure -text "ambiguous match: '$value'"
		update_from_mem $match
	    }
            "UNIQUE" -
	    "EXACT" {
		.status.line configure -text "exact match: '$value'"
		update_from_mem $match
		set cbb(mem_match) $desc
		focus [tk_focusNext %W]
		return 
	    }
	    default {
		if { $cbb(debug) } { puts "error in memorized transaction..." }
		exit
	    }
	}
	tkEntrySetCursor %W end
	set cbb(mem_match) $desc
	return
    }

    ######### bind <Control-Tab> to desc by hand
    bind .entry.line1.desc <Control-Tab> {
	if {! $cbb(use_mems) } { return }
	if {! [string length $cbb(mem_partial)] } {
	    if {! [string length $desc] } { return }
	    set cbb(mem_partial) $desc
	}

	puts $eng "find_mem $cbb(mem_entry)\t$cbb(mem_partial)"; flush $eng
	gets $eng result
	gets $eng match 
	if { $cbb(debug) } { puts $result; puts $match}

	incr cbb(mem_entry)
	update_from_mem $match

	set pieces [split $result "\t"]
	set stat  [lindex $pieces 0]

	# nothing special for responses PARTIAL, COMPLETE, UNIQUE, EXACT
	if { "$stat" == "NONE" } {
	    set desc $cbb(mem_partial)
	    set cbb(mem_entry) 0
	}
	tkEntrySetCursor %W end
	set cbb(mem_match) $desc
    }

    ######### bind <Tab> to cat
    # in this binding, $nicecat is the value seen on the screen.  
    # the internal value, $cat, is set when we focus leaves this box
    bind .entry.line2.cat <Tab> {
	
	if { $cbb(debug) } { puts "nicecat: $nicecat"}

	# skip past on these conditions
	if { "$nicecat" == "-Splits-" } { focus [tk_focusNext %W]; return }
	if {! [expr [string length $nicecat]] } { focus [tk_focusNext %W]; return }
	if {! [string compare $nicecat $cbb(cat_match)]} { 
	    if { !$cbb(cat_found) } {
		set addcat(cat) $nicecat
		set addcat(mode) "missing"
		cbbWindow.newcat
		tkwait window .newcat
		# whether or not the user adds it, they have skipped it, so skip it.
		set cbb(cat_found) 1
	    }
	    focus [tk_focusNext %W]
	    return 
	}

	# fill in match if found
	set cbb(cat_entry) 1
	set cbb(cat_found) 0
	set cbb(cat_partial) $nicecat
	puts $eng "find_cat 0\t$nicecat"; flush $eng
	gets $eng result
	gets $eng match 
	if { $cbb(debug) } { puts $result; puts $match}
	
	set pieces [split $match "\t"]
	set nicecat [lindex $pieces 1]
	
	set pieces [split $result "\t"]
	set stat  [lindex $pieces 0]
	set value [lindex $pieces 1]
	
	switch $stat {
	    "NONE" {
		.status.line configure -text "no match found."
		set nicecat $cbb(cat_partial)
	    } 
	    "PARTIAL" {
		set cbb(cat_found) 1
		.status.line configure -text "partial match: '$value'"
	    } 
	    "UNIQUE" -
	    "EXACT" {
		set cbb(cat_found) 1
		.status.line configure -text "exact match: '$value'"
		set cbb(cat_match) $nicecat
		focus [tk_focusNext %W]
		return
	    }
	    "COMPLETE" {
		set cbb(cat_found) 1
		.status.line configure -text "ambiguous match: '$value'"
	    }
	    default {
		if { $cbb(debug) } { puts "error in cat lookup..." }
		exit;
	    }
	}
	tkEntrySetCursor %W end
	set cbb(cat_match) $nicecat
	return
    }

    ### bind <Control-Tab> to cat
    # in this binding, $nicecat is the value seen on the screen.  
    # the internal value, $cat, is set when we focus leaves this box
    bind .entry.line2.cat <Control-Tab> {
	if {! [string length $cbb(cat_partial)] } { 
	    if {! [string length $nicecat] } { return }
	    set cbb(cat_partial) $nicecat
	}

	puts $eng "find_cat $cbb(cat_entry)\t$cbb(cat_partial)"; flush $eng
	gets $eng result
	gets $eng match 
	if { $cbb(debug) } { puts $result; puts $match}

	set pieces [split $match "\t"]
	set nicecat [lindex $pieces 1]

	set pieces [split $result "\t"]
	set stat  [lindex $pieces 0]
	
	incr cbb(cat_entry)
	set cbb(cat_found) 1

	# nothing special for responses PARTIAL, COMPLET, and UNIQUE
	if {"$stat" == "NONE" } {
	    set nicecat $cbb(cat_partial)
	    set cbb(cat_entry) 0
	    set cbb(cat_found) 0
	}
	tkEntrySetCursor %W end
	set cbb(cat_match) $nicecat
    }

    bind . <Alt-Home> { goto 1 }
    bind . <Meta-Home> { goto 1 }
    bind . <Alt-End> { goto [expr [listGetSize] - 1] }
    bind . <Meta-End> { goto [expr [listGetSize] - 1] }
    bind . <Up> { goto [expr [listGetCurTrans] - 2] }
    bind . <Down> { goto [expr [listGetCurTrans] + 2] }
    bind . <Prior> { 
	goto [expr [listGetCurTrans] - ($cbb(list_height) - 4)]
    }
    bind . <Next> { 
	goto [expr [listGetCurTrans] + ($cbb(list_height) - 4)]
    }
}


proc inc_check check {
    global cbb

    if { "$check" == "" } {
	set check $cbb(next_chk)
    }

    .entry.line1.check icursor end

    return [expr int($check.) + 1]
}


proc dec_check check {
    global cbb

    if { "$check" == "" } {
	set check $cbb(next_chk)
    }

    .entry.line1.check icursor end

    if { [expr $check > 1] } {
        return [expr int($check.) - 1]
    } else {
        return 1
    }
}


proc pad num {
    if { "$num" == "" } {
	set num 0
    }
    set num [expr int($num.)]
    if { [expr $num >= 0 && $num <= 9] } {
	return "0$num"
    } else {
	return $num
    }
}

