#!/usr/bin/wish -f
#  'CBB' -- Check Book Balancer
#
#   dates.tcl -- date conversion routines.
#
#  Written by Curtis Olson.  Started August 25, 1994.
#  Functions by Eugene Magnier

# define the current date format 
#
# 0 - not yet set!
# 1 - mm/dd/yyyy
# 2 - dd/mm/yyyy
# 3 - yyyy/mm/dd
# 4 - yyyy/dd/mm
# 5 - mm.dd.yyyy
# 6 - dd.mm.yyyy
# 7 - yyyy.mm.dd
# 8 - yyyy.dd.mm

proc set_date_format { format } {

    global date_format

    if {$format < 1} {
	if { $cbb(debug) } { puts "invalid date format" }
	exit
    }
    if {$format > 8} {
	if { $cbb(debug) } { puts "invalid date format $date_format" }
	exit
    }

    set date_format $format
}

proc show_date_format { } {

    global date_format

    switch $date_format {
	1 { return "mm/dd/yyyy" } 
	2 { return "dd/mm/yyyy" } 
	3 { return "yyyy/mm/dd" } 
	4 { return "yyyy/dd/mm" } 
	5 { return "mm.dd.yyyy" } 
	6 { return "dd.mm.yyyy" } 
	7 { return "yyyy.mm.dd" } 
    	8 { return "yyyy.dd.mm" }
    }
	
    if { $cbb(debug) } { puts "invalid date format $date_format" }
    exit

}

proc date_to_nicedate { date } {

    global date_format
    if {"$date" == ""} { return "" }

    if { [string length $date] == 6 } {
	if { $cbb(debug) } { puts "invalid date in update_globals()" }
	exit
    } else {
	set year [string range $date 0 3]
	set month [string range $date 4 5]
	set day [string range $date 6 7]
    }

    switch $date_format {
	1 { set nice [format "%s/%s/%s" $month $day $year] }
	2 { set nice [format "%s/%s/%s" $day $month $year]  }
	3 { set nice [format "%s/%s/%s" $year $month $day]  }
	4 { set nice [format "%s/%s/%s" $year $day $month]  }
	5 { set nice [format "%s.%s.%s" $month $day $year]  }
	6 { set nice [format "%s.%s.%s" $day $month $year]  }
	7 { set nice [format "%s.%s.%s" $year $month $day]  }
	8 { set nice [format "%s.%s.%s" $year $day $month]  }
    }
	
    return $nice
}

proc nicedate_to_date { nice current_date } {

    global date_format

    # this lets us get day, month, year without forcing the
    # separator definition.   order is defined by format
    # we allow missing entries (year, month).  any missing ones 
    # are contributed by current_date

    set cur_century [date_century $current_date)]
    set cur_year [date_year $current_date)]
    set cur_month [date_month $current_date)]
    if {! [string compare $nice ""] } { return $current_date }

    set nconv [scan $nice "%d%c%d%c%d" v1 s1 v2 s2 v3]

    switch $nconv { 
	1 -
	2 { 
	    set year $cur_year 
	    set month $cur_month 
	    set day $v1
	}
	3 -
	4 {
	    set year $cur_year
	    switch $date_format {
		1 -
		3 -
		5 -
		7 { set month $v1; set day $v2 }
		2 -
		4 -
		6 -
		8 { set month $v2; set day $v1 }
	    }
	}
	5 {
	    switch $date_format {
		1 -
		5 { set year $v3; set month $v1; set day $v2 }
		2 -
		6 { set year $v3; set month $v2; set day $v1 }
		3 -
		7 { set year $v1; set month $v2; set day $v3 }
		4 -
		8 { set year $v1; set month $v3; set day $v2 }
	    }
	    if { $year < 20 } {
		set year [expr $year + 2000]
	    }
	    if { $year < 100 } {
		set year [expr $year + 1900]
	    }
	}
    }
    set date [format "%04d%02d%02d" $year $month $day] 
    return $date
}

proc test_date { date } {
    if { [string length $date] == 6 } {
	if { $cbb(debug) } { puts "invalid date in date_month" }
	exit
    }
}    

proc date_month { date } {

    set month [string range $date 4 5]
    if { [string range $month 0 0] == "0" } {
	set month [string range $month 1 1]
    }
    return $month
}

proc date_day { date } {

    set day  [string range $date 6 7]
    if { [string range $day 0 0] == "0" } {
	set day [string range $day 1 1]
    }
    return $day
}

proc date_year { date } {

    set year [string range $date 0 3]
    return $year
}

proc date_century { date } {

    set century [string range $date 0 1]
    set century [expr $century * 100]
    return $century
}

# We now demand that year be in form YYYY
proc inc_date { date } {

    test_date $date
    set year [date_year $date]
    set month [date_month $date]
    set day [date_day $date]

    incr day

    switch $month {
	 1 { set mday 31 } 
	 2 { 
	     set mday 28 
	     if { [expr $year mod 4] == 0 } { set mday 29 }
	     if { [expr $year mod 100] == 0 } { set mday 28 }
	     if { [expr $year mod 400] == 0 } { set mday 29 }
	 }	 
	 3 { set mday 31 } 
	 4 { set mday 30 } 
	 5 { set mday 31 } 
	 6 { set mday 30 } 
	 7 { set mday 31 } 
	 8 { set mday 31 } 
	 9 { set mday 30 } 
	10 { set mday 31 } 
	11 { set mday 30 } 
	12 { set mday 31 } 
    }

    # this works for increments < $mday + $day
    if { $day > $mday } {
	set day [expr $day - $mday]
	incr month 
	if { $month == 13 } {
	    incr year
	    set month 1
	}
    }

    # why do we allow the year 10000?
    if {$year > 9999} { set year 0 }

    return "$year[pad $month][pad $day]"
}


proc dec_date date {

    test_date $date
    set year [date_year $date]
    set month [date_month $date]
    set day [date_day $date]

    # isn't there a decr function?
    set day [expr $day - 1]

    if {$day < 1} {
	switch $month {
	    1 { set mday 31 } 
	    2 { 
		set mday 28 
		if { [expr $year mod 4] == 0 } { set mday 29 }
		if { [expr $year mod 100] == 0 } { set mday 28 }
		if { [expr $year mod 400] == 0 } { set mday 29 }
	    }	 
	    3 { set mday 31 } 
	    4 { set mday 30 } 
	    5 { set mday 31 } 
	    6 { set mday 30 } 
	    7 { set mday 31 } 
	    8 { set mday 31 } 
	    9 { set mday 30 } 
	    10 { set mday 31 } 
	    11 { set mday 30 } 
	    12 { set mday 31 } 
	}
	
	# this works for decrements < $mday + $day
	set day [expr $day + $mday]
	set month [expr $month - 1]
	if { $month == 0 } {
	    set year [expr $year - 1]
	    set month 12
	}
    }

    if {$year < 0} { set year 9999 }

    return "$year[pad $month][pad $day]"
}

