#!/usr/bin/wish -f
#  'CBB' -- Check Book Balancer
#
#   file.tcl -- file management routines.
#
#  Written by Curtis Olson.  Started August 25, 1994.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# file.tcl,v 0.11 2001/08/12 20:47:36 briank Exp


#------------------------------------------------------------------------------
# File menu procedures
#------------------------------------------------------------------------------

## this function needs some work:  
# 1) if we have a group (which implies a category list), 
#    then this should be a new account in that group.
#    no need to create a new list, but need to add to the group
#    and add to the category list
# 2) if there is no defined group, we must create a group first...

# create a new account
proc acctNew {} {
    global cbb yesno argv0 eng newacct env

    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Cannot make new accounts when running in 'safe' mode." 
	tkwait window .ok
        return
    }

    if { $cbb(cur_group) == "" } { 
        cbbWindow.ok "Creating 'default' group for new account." 
	tkwait window .ok
        # Generate default group
        groupNew 1
    }

    if { [test_saved_state] } { return }

    # starting directory from current path 
    if { $cbb(cur_path) == "" } { set cbb(cur_path) "$env(HOME)/.cbb" }
    set newacct(path) "$cbb(cur_path)/"
    set newacct(file) ""
    set newacct(name) ""
    set newacct(desc) ""
    cbbWindows.newacct
    tkwait window .newacct

    if { "$newacct(file)" == "" } { return }

    # Check for existing account of same name
    if [ file exists "$cbb(cur_path)/$cbb(cur_group)/$newacct(file)" ] {
      cbbWindow.ok "Account by that name already exists."
      tkwait window .ok
      return
    }

    acctNewClear
    
    set cbb(cur_file) [file tail $newacct(file)]
    set cbb(cur_name) $newacct(name)
    set cbb(cur_desc) $newacct(desc)
    
    setMainInfo
 
    # tell engine to make the new account
    if { $cbb(debug) } { puts "make_acct $cbb(cur_name)\t$cbb(cur_path)\t$cbb(cur_file)\t$cbb(cur_desc)" }
    puts $eng "make_acct $cbb(cur_name)\t$cbb(cur_path)\t$cbb(cur_file)\t$cbb(cur_desc)"; flush $eng
    gets $eng result; 
    if { $cbb(debug) } { puts "result: $result" }
    
    if { "$result" != "ok" } {
	cbbWindow.ok "Error creating ``$cbb(cur_file)'':  $result."
	tkwait window .ok
	return
    }
    
    # re-create account list 
    updateBalanceWindow

    .status.line configure -text "Account Created"
    update
    cbbcurSave
}

# specify new account name & description
proc cbbWindows.newacct {} {
    global cbb newacct

    if {[winfo exists .newacct] == 1} {
	wm withdraw .newacct
        wm deiconify .newacct
	return                
    }

    toplevel .newacct

    option add *font $cbb(dialog_font)
    wm title .newacct "Make Account"
    wm iconname .newacct "Make Account"

    frame .newacct.frame -borderwidth 2 

    frame .newacct.frame.f
    frame .newacct.frame.n
    frame .newacct.frame.d
    frame .newacct.frame.b

    label .newacct.frame.label -text "Make New Account"

    label .newacct.frame.f.label -text "Filename"
    entry .newacct.frame.f.entry -textvariable newacct(file) -relief sunken -font $cbb(default_font)
    .newacct.frame.f.entry icursor end
    button .newacct.frame.f.browse -text "Browse.." -font $cbb(button_font) -command { 
	set result [ut:fsbox -prompt "New Account:" -cancelvalue "" -grab 1 -master . -dir $newacct(path)]
	set newacct(file) [file tail $result]
    }

    label .newacct.frame.n.label -text "Nickname"
    entry .newacct.frame.n.entry -textvariable newacct(name) -relief sunken -font $cbb(default_font)

    label .newacct.frame.d.label -text "Description"
    entry .newacct.frame.d.entry -textvariable newacct(desc) -relief sunken -font $cbb(default_font)

    button .newacct.frame.b.create -text "Create Account" -font $cbb(button_font) -command { destroy .newacct }
    button .newacct.frame.b.cancel -text "Cancel" -font $cbb(button_font) -command { 
	set newacct(name) ""
	set newacct(file) ""
	set newacct(desc) ""
	destroy .newacct
    }

    pack .newacct.frame -fill both -expand 1 -padx 4 -pady 4
    pack .newacct.frame.label -padx 4 -pady 2

    pack .newacct.frame.f .newacct.frame.n .newacct.frame.d -fill both -expand 1 -padx 4 -pady 2 
    pack .newacct.frame.b -side bottom -fill x -padx 4 -pady 2 

    pack .newacct.frame.f.label -side left
    pack .newacct.frame.f.browse -side right
    pack .newacct.frame.f.entry -side left -fill x -expand 1

    pack .newacct.frame.n.label -side left
    pack .newacct.frame.n.entry -side left -fill x -expand 1

    pack .newacct.frame.d.label -side left
    pack .newacct.frame.d.entry -side left -fill x -expand 1

    pack .newacct.frame.b.create .newacct.frame.b.cancel -side left -fill x -expand 1 -padx 4 -pady 4

    focus .newacct.frame.n.entry 
}

proc acctNewClear {} {
    global cbb eng argv0

    if { $cbb(debug) } { puts "beginning of acctNewClear" }

    # clear our list box
    .trans.list configure -state normal
    .trans.list delete 0.0 end
    .trans.list configure -state disabled

    # tell engine to clear transactions
    puts $eng "init_trans"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts "init_trans(): $result" }

    set cbb(cur_file) ""
    set cbb(cur_name) ""
    set cbb(cur_desc) ""
    setMainInfo

    clear_entry_area
}

# Dialog for renaming/moving accounts
# Sets newacct(group), newacct(name), newacct(desc)
proc cbbWindows.renameAcct {} {
    global cbb newacct

    if {[winfo exists .renameAcct] == 1} {
	wm withdraw .renameAcct
        wm deiconify .renameAcct
	return                
    }

    toplevel .renameAcct

    option add *font $cbb(dialog_font)
    wm title .renameAcct "Rename/Move Account"
    wm iconname .renameAcct "Rename/Move Account"

    frame .renameAcct.frame -borderwidth 2 

    # Frame for (new) group name
    frame .renameAcct.frame.g
    # Frame for (new) account name
    frame .renameAcct.frame.n
    # Frame for (new) account desc
    frame .renameAcct.frame.d
    # Frame for buttons
    frame .renameAcct.frame.b

    label .renameAcct.frame.label -text "Rename/Move Account"

    label .renameAcct.frame.g.label -text "New group"
    entry .renameAcct.frame.g.entry -textvariable newacct(group) -relief sunken -font $cbb(default_font)

    label .renameAcct.frame.n.label -text "New account name"
    entry .renameAcct.frame.n.entry -textvariable newacct(name) -relief sunken -font $cbb(default_font)

    label .renameAcct.frame.d.label -text "New description"
    entry .renameAcct.frame.d.entry -textvariable newacct(desc) -relief sunken -font $cbb(default_font)

    button .renameAcct.frame.b.create -text "Continue" -font $cbb(button_font) -command { destroy .renameAcct }
    button .renameAcct.frame.b.cancel -text "Cancel" -font $cbb(button_font) -command { 
        set newacct(group) ""
	set newacct(name) ""
	set newacct(desc) ""
	destroy .renameAcct
    }

    pack .renameAcct.frame -fill both -expand 1 -padx 4 -pady 4
    pack .renameAcct.frame.label -padx 4 -pady 2

    pack .renameAcct.frame.g .renameAcct.frame.n .renameAcct.frame.d -fill both -expand 1 -padx 4 -pady 2 
    pack .renameAcct.frame.b -side bottom -fill x -padx 4 -pady 2 

    pack .renameAcct.frame.g.label -side left
    pack .renameAcct.frame.g.entry -side left -fill x -expand 1

    pack .renameAcct.frame.n.label -side left
    pack .renameAcct.frame.n.entry -side left -fill x -expand 1

    pack .renameAcct.frame.d.label -side left
    pack .renameAcct.frame.d.entry -side left -fill x -expand 1

    pack .renameAcct.frame.b.create .renameAcct.frame.b.cancel -side left -fill x -expand 1 -padx 4 -pady 4

    focus .renameAcct.frame.n.entry 
}

proc acctClear {} {
    global cbb eng argv0

    if { $cbb(debug) } { puts "beginning of acctClear" }

    # clear our list box
    .trans.list configure -state normal
    .trans.list delete 0.0 end
    .trans.list configure -state disabled

    setMainInfo
    
    clear_entry_area
}

# Dialog for creating new group 
proc cbbWindows.groupNew {} {
    global cbb newacct

    if {[winfo exists .groupNew] == 1} {
	wm withdraw .groupNew
        wm deiconify .groupNew
	return                
    }

    toplevel .groupNew

    option add *font $cbb(dialog_font)
    wm title .groupNew "Make New Group"
    wm iconname .groupNew "Make New Group"

    frame .groupNew.frame -borderwidth 2 

    # Frame for (new) group name
    frame .groupNew.frame.g
    # Frame for buttons
    frame .groupNew.frame.b

    label .groupNew.frame.label -text "Make New Group"

    label .groupNew.frame.g.label -text "New group"
    entry .groupNew.frame.g.entry -textvariable newacct(group) -relief sunken -font $cbb(default_font)

    button .groupNew.frame.b.create -text "Continue" -font $cbb(button_font) -command { destroy .groupNew }
    button .groupNew.frame.b.cancel -text "Cancel" -font $cbb(button_font) -command { 
        set newacct(group) ""
	destroy .groupNew
    }

    pack .groupNew.frame -fill both -expand 1 -padx 4 -pady 4
    pack .groupNew.frame.label -padx 4 -pady 2

    pack .groupNew.frame.g -fill both -expand 1 -padx 4 -pady 2 
    pack .groupNew.frame.b -side bottom -fill x -padx 4 -pady 2 

    pack .groupNew.frame.g.label -side left
    pack .groupNew.frame.g.entry -side left -fill x -expand 1

    pack .groupNew.frame.b.create .groupNew.frame.b.cancel -side left -fill x -expand 1 -padx 4 -pady 4

    focus .groupNew.frame.g.entry 
}


# gui interaction to load account
proc acctLoad {} {
    global cbb yesno

    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Cannot load accounts when running in 'safe' mode." 
	tkwait window .ok
        return
    }

    if { [test_saved_state] } { return }

    option add *font $cbb(default_font)

    set select_result [ut:fsbox -prompt "Load Account:" -cancelvalue "" -grab 1 -master . -dir $cbb(cur_path) ]
    if { "$select_result" == "" } { return }    

    set result [load_account $cbb(cur_group) $select_result]
    if {$result == "error"} {
	if {$cbb(cur_file) == ""} {return}
	set result [load_account $cbb(cur_group) $cbb(cur_path)/$cbb(cur_group)/$cbb(cur_file)];
	if {"$result" == "error"} { 
	    cbbWindow.ok "can't reload original account $cbb(cur_path)/$cbb(cur_group)/$cbb(cur_file)"
	    tkwait window .ok
	    return
	}
        cbbcurSave
    }
}

# Save transactions only, not associated group and category info
# You probably want to use acctSave, not transSave.
proc transSave { filename accountname group } {
    global cbb argv0 eng

    setMainInfo

    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Cannot save when running in 'safe' mode." 
	tkwait window .ok
        return
    }
    
    if { $filename == "" } { return }

    
    # tell engine to save transactions (switch to waiting cursor) 
    .status.line configure -text "Saving transactions to $cbb(cur_path)/$group/$filename"
    set_cursor_wait 1
    update

    puts $eng "save_trans $filename\t$accountname\t$group"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts "saving transactions ... $result" }

    if { "$result" != "ok" } {
        cbbWindow.ok "Error saving $cbb(cur_path)/$group/$filename:  $result."
        tkwait window .ok
	set_cursor_wait 0
	return
    }

    cbbcurSave

    # ok we are clean again
    acctSetClean

    set_cursor_wait 0
}

proc acctSave {} {
    global cbb argv0 eng

    # tell engine to save transactions (switch to waiting cursor) 
    transSave $cbb(cur_file) $cbb(cur_name) $cbb(cur_group)

    # tell engine to save categories
    puts $eng "save_cats $cbb(cur_path)"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts "saving categories ... $result" }

    if { "$result" != "ok" } {
        cbbWindow.ok "Error saving categories:  $result."
        tkwait window .ok
	set_cursor_wait 0
	return
    }

    # tell engine to save account list
    puts $eng "save_accts"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts "saving accounts ... $result" }

    if { "$result" != "ok" } {
        cbbWindow.ok "Error saving accounts:  $result."
        tkwait window .ok
	set_cursor_wait 0
	return
    }

    cbbcurSave

    # ok we are clean again
    acctSetClean

    set_cursor_wait 0
}


proc acctRename {} {
    global cbb eng newacct

    option add *font $cbb(default_font)

    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Cannot save as when running in 'safe' mode." 
        tkwait window .ok
        return
    }

    if { $cbb(cur_file) == "" } { 
        cbbWindow.ok "No account selected." 
	tkwait window .ok
        return
    }

    set oldname $cbb(cur_name)
    set oldfile $cbb(cur_file)
    set oldpath $cbb(cur_path)
    set oldgroup $cbb(cur_group)
    set newacct(path) "$cbb(cur_path)/"
    set newacct(file) "$cbb(cur_file)"
    set newacct(name) "$cbb(cur_name)"
    set newacct(desc) "$cbb(cur_desc)"
    set newacct(group) "$cbb(cur_group)"
    cbbWindows.renameAcct
    tkwait window .renameAcct

    if { "$newacct(name)" == "" } { return }
    if { "$newacct(group)" == "" } { return }

    # Save to new account file
    if { $cbb(debug) } { puts "acctRename(): transSave $newacct(file) $newacct(name) $newacct(group)" }
    transSave $newacct(file) $newacct(name) $newacct(group)

    set cbb(cur_file) "$newacct(name)"
    set cbb(cur_name) "$newacct(name)"
    set cbb(cur_desc) "$newacct(desc)"
    set cbb(cur_group) "$newacct(group)"

    # Modify group, category info (WARNING: May load new group!)
    if { $cbb(debug) } { puts "acctRename(): rename_acct $oldname\t$cbb(cur_name)\t$cbb(cur_group)\t$cbb(cur_desc)"; }
    puts $eng "rename_acct $oldname\t$cbb(cur_name)\t$cbb(cur_group)\t$cbb(cur_desc)"; flush $eng
    gets $eng result;
    if { $cbb(debug) } { puts "result: $result" }
    if { "$result" != "ok" } {
	cbbWindow.ok "Error creating ``$cbb(cur_file)'':  $result."
	tkwait window .ok
	return
    }

    # Save all information, since group may have changed
    acctSave

    # Remove old account file
    set fileToDel "$oldpath/$oldgroup/$oldfile"
    if [ file exists "$fileToDel" ] {
      if { $cbb(debug) } { puts "acctRename(): deleting $fileToDel" }
      file delete "$fileToDel"
    } else {
	cbbWindow.ok "Error removing $fileToDel during rename/move."
	tkwait window .ok
	return
    }

    # re-create account list 
    if { $cbb(debug) } { puts "acctRename(): updateBalanceWindow" }
    updateBalanceWindow

    .status.line configure -text "Account Renamed -- $cbb(cur_file)"
    .status.files.acct configure -text "account: $cbb(cur_file)"
    setMainInfo

    if { $cbb(debug) } { puts "acctRename(): update" }
    update
    if { $cbb(debug) } { puts "acctRename(): cbbcurSave" }
    cbbcurSave
}

# Select a file to import
proc acctImport {mode} {
    global cbb

    option add *font $cbb(default_font)

    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Cannot import when running in 'safe' mode." 
        tkwait window .ok
        return
    }

    if { $cbb(cur_name) == "" } {
        cbbWindow.ok "Please create a new account prior to importing." 
        tkwait window .ok
        return
    }

    switch $mode {
	"cbb9" { set prompt "CBB 0.9 Import:" } 
	"cbb8" { set prompt "CBB 0.8 Import:" } 
	"qifA"  { set prompt "QIF Import Account:" }
	"qifG"  { set prompt "QIF Import Group:" }
	default { return }
    }

    set select_result [ut:fsbox -prompt $prompt -cancelvalue "" -grab 1 -master .  -dir "$cbb(cur_path)/.cbb" ]

    if { "$select_result" != "" } {
	acctImportFile $select_result $mode
    }

    # Update acct balance window
    updateBalanceWindow
}


# Do the actual import
proc acctImportFile args {
    global cbb eng key cat
    global nicecat com cleared total

    set file [lindex $args 0]
    set type [lindex $args 1]

    set_cursor_wait 1
    .status.line configure -text "Importing transactions from [file tail $file]."
    update

    if { $cbb(debug) } { puts "Importing data file $file" }

    switch $type { 
	"cbb9" { set command "import_cbb9" }
	"cbb8" { set command "import_cbb8" }
	"qifA" { set command "import_qif_account" }
	"qifG" { set command "import_qif_group" }
    }

    puts $eng "$command $file"; flush $eng
    if { $cbb(debug) } { puts "Reading result" }
    gets $eng result
    if { $cbb(debug) } { puts "Got result: $result" }
    if { [string range $result 0 6] == "error" } {
	cbbWindow.ok "msg from import: $result"
	tkwait window .ok
    }

    if { $type == "qifG" } {
	# re-load the account names
        updateBalanceWindow

	# set correct values for current name, file, etc
	puts $eng "current_acct"; flush $eng
	gets $eng line
	if { $line == "error" } { 
	    .trans.list configure -state normal
	    .trans.list delete 0.0 end
	    clear_entry_area
	    set_cursor_wait 0
	    return 
	}
	set pieces [split $line "\t"]
	set cbb(cur_name)  [lindex $pieces 0]
	set cbb(cur_file)  [lindex $pieces 1]
	set cbb(cur_desc)  [lindex $pieces 2]
    }

    # clear and re-load transaction list
    .trans.list configure -state normal
    .trans.list delete 0.0 end

    set N 0
    puts $eng "all_trans"; flush $eng
    gets $eng line1
    while { $line1 != "none" } {
	if { $cbb(debug) } { puts "line1 = $line1" }
	gets $eng line2
	listAddTrans $line1 $line2
	incr N
        gets $eng line1
    }
    .trans.list configure -state disabled

    # set listbox view to end
    goto [listGetSize]
    clear_entry_area
    set_cursor_wait 0

    setMainInfo
    if { $type != "qifG" } { acctSetDirty }
}


# Select a file to export to
proc acctExport {mode} {
    global cbb env

    option add *font $cbb(default_font)

    if { $cbb(cur_file) == "" } {
        cbbWindow.ok "No account opened for export." 
        tkwait window .ok
        return
    }

    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Cannot import when running in 'safe' mode." 
        tkwait window .ok
        return
    }

    switch $mode {
	"cbb9" { set prompt "CBB 0.9 Export:" } 
	"cbb8" { set prompt "CBB 0.8 Export:" } 
	"qif"  { set prompt "QIF Export:" }
	default { return }
    }

    set select_result [ut:fsbox -prompt $prompt -cancelvalue "" -grab 1 -master .  -dir "$env(HOME)/.cbb" ]

    if { $cbb(debug) } { puts "After select: Export [file tail $select_result]"}

    if { "$select_result" != "" } {
	acctExportFile $select_result $mode
    }
}

# Update all entries in the lower balance window.  This could probably
# benefit from updating only the current account line...
proc updateBalanceWindow {} {
  global eng 

  .acct.list delete 0 [.acct.list size]
  # Create headers
  .acct.list insert end [format "%-14s %-20s %-10s %-10s %-10s" "Acct name" "Desc" "Cur Bal" "End Bal" "Cleard Bal"]
  puts $eng "all_accts"; flush $eng
  gets $eng line
  while { $line != "none" } {
      # need to fix format on basis of what is actually returned...
      set pieces [split $line "\t"]
      set name  [lindex $pieces 0]
      set desc  [lindex $pieces 2]
      set curbal   [lindex $pieces 3]
      set endbal  [lindex $pieces 4]
      set clearbal [lindex $pieces 5]
      .acct.list insert end [format "%-14s  %-20.20s  %10.2f  %10.2f %10.2f" $name $desc $curbal $endbal $clearbal]
      gets $eng line
  }
}

# Do the actual export
proc acctExportFile args {
    # I have removed global entries which are no longer used - EAM
    global cbb eng key cat
    global nicecat com cleared total

    set file [lindex $args 0]
    set type [lindex $args 1]

    .status.line configure -text "Exporting transactions to [file tail $file]."
    update

    if { $cbb(debug) } { puts "Exporting data file $file" }

    switch $type { 
	"cbb9" {
	    puts $eng "export_cbb9 $file"; flush $eng
	    gets $eng result
	} 
	"cbb8" {
	    puts $eng "export_cbb8 $file"; flush $eng
	    gets $eng result
	} 
	"qif" {
	    puts $eng "export_qif $file"; flush $eng
	    gets $eng result
	}
    }

    .status.line configure -text "Exported transactions to [file tail $file]: $result."

}

proc cbbQuit {} {
    global cbb yesno

    if {  $cbb(safe_mode) } {
	cbbWindow.ok "Exiting from CBB in safe mode - no changes are saved."
	tkwait window .ok
	exit
    }

    if { [test_saved_state] } { return }

    if { $cbb(pref_chg) } {
	cbbWindow.yesno "You have not saved your current preferences. \
		Would you like to save before quitting?"
	tkwait window .yesno
	
	switch $yesno(result) {
	    "yes" { cbbrcSave } 
	    "no"  { }
	    "cancel" { return } 
	}
    }

    exit
}


# write out current ~/.cbbcur.tcl file
proc cbbcurSave {} {
    global cbb

    if { $cbb(safe_mode) == 1 } {
	return
    }

    set now_today [get_sys_date]

    set rchandle [open $cbb(save_cur_file) w]
    puts $rchandle "# ~/.cbbcur.tcl"
    puts $rchandle "#"
    puts $rchandle "# Created by CBB $cbb(version)"
    puts $rchandle "# Created on $now_today"
    puts $rchandle "#"
    puts $rchandle "# This is machine generated tcl code.  Please use caution"
    puts $rchandle "# when hand editing!"
    puts $rchandle ""
    puts $rchandle "# File to load when CBB initially starts."
    puts $rchandle "set cbb(cur_file) $cbb(cur_file)"
    puts $rchandle ""
    puts $rchandle "# Current directory when CBB initially starts."
    puts $rchandle "set cbb(cur_path) $cbb(cur_path)"
    puts $rchandle ""
    puts $rchandle "# Current group when CBB initially starts."
    puts $rchandle "set cbb(new_group) $cbb(cur_group)"

    close $rchandle
}

# write out current ~/.cbbrc.tcl file
proc cbbrcSave {} {
    global cbb

    set cbb(pref_chg) 0

    if { $cbb(safe_mode) == 1 } {
	return
    }

    set now_today [get_sys_date]

    set rchandle [open $cbb(conf_file) w]
    puts $rchandle "# ~/.cbbrc.tcl"
    puts $rchandle "#"
    puts $rchandle "# Created by CBB $cbb(version)"
    puts $rchandle "# Created on $now_today"
    puts $rchandle "#"
    puts $rchandle "# This is machine generated tcl code.  Please use caution"
    puts $rchandle "# when hand editing!"
    puts $rchandle ""
    puts $rchandle "# Debugging:  0 = disabled, 1 = enabled"  
    puts $rchandle "set cbb(debug) $cbb(debug)"
    puts $rchandle ""
    puts $rchandle "# Enable Balloon Help:  0 = disabled, 1 = enabled"
    puts $rchandle "set cbb(balloon,on) $cbb(balloon,on)"
    puts $rchandle ""
    puts $rchandle "# Enable Splash Screen:  0 = disabled, 1 = enabled"
    puts $rchandle "set cbb(splash) $cbb(splash)"
    puts $rchandle ""
    puts $rchandle "# Auto-highlight entry fields:  0 = disabled, 1 = enabled"  
    puts $rchandle "set cbb(auto_hilite) $cbb(auto_hilite)"
    puts $rchandle ""
    puts $rchandle "# Use Memorized Transactions:  0 = disabled, 1 = enabled"
    puts $rchandle "set cbb(use_mems) $cbb(use_mems)"
    puts $rchandle ""
    puts $rchandle "# Date Format:  1=USA 2=International"
    puts $rchandle "set cbb(date_fmt) $cbb(date_fmt)"
    puts $rchandle ""
    puts $rchandle "# Auto Save Interval (in milleseconds)"
    puts $rchandle "set cbb(save_interval) $cbb(save_interval)"
    puts $rchandle "# Auto-save when switching files or quitting"
    puts $rchandle "set cbb(auto_save) $cbb(auto_save)"
    puts $rchandle ""
    puts $rchandle "# Main List Box Dimensions"
    puts $rchandle "set cbb(list_width) $cbb(list_width)"
    puts $rchandle "set cbb(list_height) $cbb(list_height)"
    puts $rchandle ""
    puts $rchandle "# Height of account list box"
    puts $rchandle "set cbb(acctlist_height) $cbb(acctlist_height)"
    puts $rchandle ""
    puts $rchandle "# Maximum splits per entry"
    puts $rchandle "set cbb(max_splits) $cbb(max_splits)"
    puts $rchandle ""
    puts $rchandle "# Startdate ..."
    puts $rchandle "set cbb(use_sdate) $cbb(use_sdate)"
    puts $rchandle "set cbb(sdate) \"$cbb(sdate)\""
    puts $rchandle "set cbb(report_sdate) \"$cbb(report_sdate)\""
    puts $rchandle ""
    puts $rchandle "# Account Cache ..."
    puts $rchandle "set cbb(cache) $cbb(cache)"
    puts $rchandle ""
    puts $rchandle "# Cryptography ..."
    puts $rchandle "set cbb(use_crypt) $cbb(use_crypt)"
    puts $rchandle "set cbb(encrypt) \"$cbb(encrypt)\""
    puts $rchandle "set cbb(decrypt) \"$cbb(decrypt)\""
    puts $rchandle ""
    puts $rchandle "# Duplicate Entries ..."
    puts $rchandle "set cbb(duplicate) $cbb(duplicate)"
    puts $rchandle "set cbb(cmpfields) $cbb(cmpfields)"
    puts $rchandle ""
    puts $rchandle "# Fonts ..."
    puts $rchandle "set cbb(msg_text_font) \"$cbb(msg_text_font)\""
    puts $rchandle "set cbb(button_font) \"$cbb(button_font)\""
    puts $rchandle "set cbb(fixed_header_font) \"$cbb(fixed_header_font)\""
    puts $rchandle "set cbb(fixed_font) \"$cbb(fixed_font)\""
    puts $rchandle "set cbb(status_line_font) \"$cbb(status_line_font)\""
    puts $rchandle "set cbb(menu_font) \"$cbb(menu_font)\""
    puts $rchandle "set cbb(dialog_font) \"$cbb(dialog_font)\""
    puts $rchandle "set cbb(default_font) \"$cbb(default_font)\""
    puts $rchandle ""
    puts $rchandle "# Colors ..."
    puts $rchandle "set cbb(head_color) \"$cbb(head_color)\""
    puts $rchandle "set cbb(hilite_color) \"$cbb(hilite_color)\""
    puts $rchandle "set cbb(list_line1_color) \"$cbb(list_line1_color)\""
    puts $rchandle "set cbb(list_line2_color) \"$cbb(list_line2_color)\""
    puts $rchandle "set cbb(trans_text_color) \"$cbb(trans_text_color)\""
    puts $rchandle "set cbb(trans_neg_color) \"$cbb(trans_neg_color)\""
    puts $rchandle "set cbb(future_color) \"$cbb(future_color)\""
    puts $rchandle "set cbb(future_neg_color) \"$cbb(future_neg_color)\""
    puts $rchandle ""
    puts $rchandle "# External web browser"
    puts $rchandle "set cbb(web_browser) \"$cbb(web_browser)\""

    close $rchandle
}

# create a new group
# If an argument of 1 is passed, generate a group called "default" instead.
proc groupNew { { createDefaultGroup 0 } } {
    global cbb yesno argv0 eng newacct env

    if { !$createDefaultGroup } {
        if { $cbb(safe_mode) == 1 } {
            cbbWindow.ok "Safe mode" 
            tkwait window .ok
            return
        }
        if { [test_saved_state] } { return }

        # Right now, the CBB root path is hard-coded.  This really needs to be a
        # user-settable option! 
        if { $cbb(cur_path) == "" } { set cbb(cur_path) "$env(HOME)/.cbb" }

        # Display dialog
        cbbWindows.groupNew
        tkwait window .groupNew

        # Compare entered value to list of groups.  If it exists, warn user that group
        # already exists.
        set groupList [ getGroups ]
        if { [ lsearch -exact $groupList $newacct(group) ] != -1 } {
          cbbWindow.ok "Group already exists."
          tkwait window .ok
          return
        }

        set newname "$env(HOME)/.cbb/$newacct(group)"
    } else {
        set newname "$env(HOME)/.cbb/default"
    }

    if { $newname == "" } { return }
    set cbb(cur_path) [file dirname $newname]
    set cbb(cur_group) [file tail $newname]
    set cbb(cur_cats) $cbb(cur_group) 

    acctNewClear
    
    # tell engine to make the new group
    if { [put_eng "make_group $cbb(cur_path)\t$cbb(cur_group)"] } { return }
    
    # tell engine to make the new categories
    if { [put_eng "make_cats $cbb(cur_path)\t$cbb(cur_cats)"] } { return }
    
    # re-create account list 
    updateBalanceWindow

    .status.line configure -text "New Group -- $cbb(cur_group)"
    .status.files.group configure -text "group: $cbb(cur_group)"
    update
    
}

# gui interaction to load a group
proc groupLoad {} {
    global cbb yesno argv0 eng newacct env
    
    if { $cbb(safe_mode) == 1 } {
        cbbWindow.ok "Safe mode" 
	tkwait window .ok
        return
    }
    if { [test_saved_state] } { return }
    
    # starting directory from current path 
    if { $cbb(cur_path) == "" } { set cbb(cur_path) "$env(HOME)/.cbb" }
    set newname [ut:fsbox -prompt "Load Group:" -cancelvalue "" -grab 1 -master . -dir $cbb(cur_path) ]
    if { $newname == "" } { return }
    set newname [ file tail $newname ]
    # Strip off the .grp suffix returned from the fsbox
    regexp "(.*)\.grp$" $newname match group
    set result [load_group $cbb(cur_path) $group ]
    # load_group assigns the correct values to $cbb(cur_path) & $cbb(cur_group)
    if {"$result" == "error"} { 
	if {$cbb(cur_group) == ""} { return }
	set result [load_group $cbb(cur_path) $cbb(cur_group)]
	if {"$result" == "error"} { 
	    cbbWindow.ok "can't reload original group $cbb(cur_path)/$cbb(cur_group)"
	    tkwait window .ok
	    return
	}
	return
    }
    acctClear

    .status.line configure -text "Group -- $cbb(cur_group)"
    .status.files.group configure -text "group: $cbb(cur_group)"
    update
    
}

# save the current file at regular intervals in case the computer crashes
# or something
proc setup_auto_save {} {
    global cbb eng

    if { $cbb(safe_mode) == 1 } { return }

    after $cbb(save_interval) {
	if { [acctIsDirty] } {
	    set auto_save_file "$cbb(cur_path)/#$cbb(cur_file)#"
	    .status.line configure -text "Auto saving to [file tail $auto_save_file]"
	    update
	    if { $cbb(debug) } {
		puts "Auto saving to [file tail $auto_save_file]"
	    }
	    
	    # tell engine to save transactions
	    # puts $eng "auto_save_trans $auto_save_file"
	    # flush $eng
	}
    
	# reset auto save
	setup_auto_save
    }
}


# load account file -- check things like the group, etc...
# this function loads in a cbb account file, providing consitency checks
# we need to add some conditions for the cases of 1) the 
# group already loaded, just a new file (check group, don't load
# if the same), 2) errors in the load process

# input is complete path to the file
# Note that we now have to know which group the account belongs to, since different
# groups can have the same account names.
proc load_account {group file} {
    global cbb eng argv0 start_pos

    if { $cbb(debug) } { puts "load_account( $group $file )"; }

    set check_count 0
    set nocheck_count 0

    # no file selected
    if { "$file" == "" } { return "none" }

    if { ! [file exists $file] } {
	cbbWindow.ok "file $file not found"
	tkwait window .ok
        return "error"
    }	

    ### this section gets the perl engine set up

    # check if this is a new group, load in group & category info 
    if { "$group" != "$cbb(cur_group)" } {
        if { $cbb(debug) } { puts "load_account(): loading group, cat info" }
	# (ask for confirmation?)
	set result [load_group $cbb(cur_path) $group]
	if {"$result" == "error"} { 
	    cbbWindow.ok "can't load new group for root dir $cbb(cur_path) and group $group"
	    tkwait window .ok
	    return "error"
	}
    }


    ####### this breaks with relative paths.  
    # load account in engine
    set base [file tail $file]
    if { $cbb(debug) } { puts "load_trans $base" }
    puts $eng "load_trans $base"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }
    if { "$result" == "error" } { 
	cbbWindow.ok "error loading transactions from file"
	tkwait window .ok
	return "error"
    }
    
    ### at this point, the perl engine has loaded the appropriate files

    acctClear
    
    # change window attributes
    . configure -cursor watch
    .trans.list configure -cursor watch
    .status.line configure -text "Loading transactions from [file tail $file]"
    update

    # load the transaction list from perl engine
    .trans.list configure -state normal
    set start_pos 0
    if { $cbb(use_sdate) } {
	puts $eng "part_trans $cbb(int_sdate)"
    } else {
    	puts $eng "all_trans"
    }
    flush $eng
    gets $eng line1
    while { $line1 != "none" } {
	gets $eng line2
	listAddTrans $line1 $line2

	# leave for now, but shouldn't this be a perl function call? EAM
	set check [string trim [string range $line1 0 4]]
	if { "$check" != "" } {
	    set cbb(next_chk) $check
	    incr check_count
	} else {
	    incr nocheck_count
	}

        gets $eng line1
    }
    .trans.list configure -state disabled

    if { $nocheck_count > $check_count } {
	set cbb(entry_area_focus) .entry.line1.date
    } else {
	set cbb(entry_area_focus) .entry.line1.check
    }

    # set listbox view to start position
    goto $start_pos
    clear_entry_area

    . configure -cursor left_ptr
    .trans.list configure -cursor xterm

    # hash out the memorized transactions
    if { $cbb(debug) } { puts "Hashing memorized transactions" }
    puts $eng "rehash_mems"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts "result: $result" }
    # no error condition defined

    # find account detail on basis of filename
    puts $eng "find_acct_file $base"; flush $eng
    gets $eng result
    if { $result == "error"} {
	cbbWindow.ok "Error getting account information for $base"
	tkwait window .ok
	return "error"
    }

    set pieces [split $result "\t"]
    set cbb(cur_name)  [lindex $pieces 0]
    set cbb(cur_file)  [lindex $pieces 1]
    set cbb(cur_desc)  [lindex $pieces 2]

    setMainInfo
    .status.line configure -text "Loaded Account"

    acctSetClean
}

# load a group
proc load_group {rootPath group} {
    global cbb eng

    set path $rootPath  
    set file $group

    # Clear all transactions, reset internal vars
    acctNewClear

    # init group
    if { $cbb(debug) } { puts "load_group()...initializing group" }
    puts $eng "init_accts"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }
    if { "$result" == "error" } { return "error" }
    
    # load group
    if { $cbb(debug) } { puts "load_group()...loading group: root path=$path group=$file" }
    puts $eng "load_accts $path\t$file"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts "load_group()...result from load_accts call $result" }
    if { "$result" == "error 1" } { 
	cbbWindow.ok "group file for root path $path and group $file not found" 
	tkwait window .ok
	return "error";
    }
    if { "$result" == "error 2" } { 
	cbbWindow.ok "file at root path $path and group $group is not a cbb group file" 
	tkwait window .ok
	return "error";
    }
    if { "$result" != "ok" } { 
	cbbWindow.ok "error loading group file for root path $path and group $file" 
	tkwait window .ok
	return "error";
    }
    
    # get categories
    puts $eng "def_cats"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }
    if { "$result" == "error" } { return "error" }
    set cbb(cur_cats) $result
    
    # get categories
    puts $eng "load_cats $cbb(cur_path)\t$cbb(cur_cats)"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }
    if { "$result" == "error" } { return "error" }
    
    # check category entries against account list
    puts $eng "check_cats_accounts"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }
#    if { "$result" == "error" } { 
#	# fix_cats_accounts pops up a window to decide how to treat missing account cats
#	set choice [fix_cats_accounts]
#	if { "$choice" == "cancel" } { return "error" }
#    }

    # load the account names (if needed -- not needed for old group)
    updateBalanceWindow

    set cbb(cur_path) $path
    set cbb(cur_group) $file

    .status.line configure -text "Group -- $cbb(cur_group)"
    .status.files.group configure -text "group: $cbb(cur_group)"

    return "ok"

}

proc fix_cats_accounts { } {
    return "cancel"
}

proc set_cursor_wait { state } {

    if { $state == 1 } {
	. configure -cursor watch
	.trans.list configure -cursor watch
    } else {
	. configure -cursor left_ptr
	.trans.list configure -cursor xterm
    }

}

# Get list of groups
proc getGroups {} {
  global cbb
  set pwd [ mypwd ]
  if [ catch { cd $cbb(cur_path) } err ] {
    if { $cbb(debug) } { puts "getGroups(): result=$err" }
    return $err
  }
  foreach match [ glob -nocomplain * ] {
    if [ file isdirectory $match ] {
      lappend groupList $match
    }
  }
  cd $pwd
  if { $cbb(debug) } { puts "getGroup(): groups found: $groupList" }
  if ![ info exists groupList ] { set groupList {} }
  return $groupList
}
